<?php
/**
 * Plugin Name: Mahn & Rot - Inkasso für WooCommerce
 * Plugin URI:  https://www.mahn-rot.de
 * Description: A plugin that provides a custom admin page under WooCommerce to select orders and send them to Mahn & Rot via their PHP API classes, with an integrated API credentials form. Uses a custom status "M&R Inkasso."
 * Version:     1.0.0
 * Author:      Mahn & Rot GmbH
 * Author URI:  https://www.mahn-rot.de/woocommerce
 * Text Domain: mahn-rot-rechnungsbearbeiter
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

// Adjust the path if needed
require_once plugin_dir_path( __FILE__ ) . 'MahnRot_API.class.php';
require_once plugin_dir_path( __FILE__ ) . 'MahnRot_Email_Helper.php';

class MahnRotRechnungsbearbeiterCustomPage {

    private $orders_per_page = 10;

    public function __construct() {
        // Check if WooCommerce is active
        add_action( 'plugins_loaded', array( $this, 'check_woocommerce_active' ) );

        // Hook to save the API credentials
        add_action( 'admin_post_save_mahn_rot_api_settings', array( $this, 'save_api_credentials' ) );

        // Register the custom order status (M&R Inkasso)
        add_action( 'init', array( $this, 'register_mr_inkasso_status' ) );

        // Add it to the WC order status list
        add_filter( 'wc_order_statuses', array( $this, 'add_mr_inkasso_to_order_statuses' ) );
    }

    /**
     * Registers the custom status "wc-mr-inkasso".
     */
    public function register_mr_inkasso_status() {
        register_post_status( 'wc-mr-inkasso', array(
            'label'                     => _x( 'M&R Inkasso', 'Order status', 'mahn-rot-rechnungsbearbeiter' ),
            'public'                    => true,
            'exclude_from_search'       => false,
            'show_in_admin_all_list'    => true,
            'show_in_admin_status_list' => true,
            'label_count'               => _n_noop( 'M&R Inkasso <span class="count">(%s)</span>', 'M&R Inkasso <span class="count">(%s)</span>', 'mahn-rot-rechnungsbearbeiter' ),
        ) );
    }

    /**
     * Adds our custom status to the list of WooCommerce statuses.
     */
    public function add_mr_inkasso_to_order_statuses( $order_statuses ) {
        // Insert our new status after "cancelled" (arbitrary choice)
        $new_statuses = array();
        foreach ( $order_statuses as $key => $label ) {
            $new_statuses[ $key ] = $label;
            if ( 'wc-cancelled' === $key ) {
                $new_statuses['wc-mr-inkasso'] = _x( 'M&R Inkasso', 'Order status', 'mahn-rot-rechnungsbearbeiter' );
            }
        }
        return $new_statuses;
    }

    /**
     * Check if WooCommerce is active.
     */
    public function check_woocommerce_active() {
        if ( ! class_exists( 'WooCommerce' ) ) {
            add_action( 'admin_notices', array( $this, 'woocommerce_missing_notice' ) );
            return;
        }

        // If WooCommerce is active, set up sub-menu & post handler
        add_action( 'admin_menu', array( $this, 'register_admin_menu' ) );
        add_action( 'admin_post_mahn_rot_send_orders', array( $this, 'handle_send_orders' ) );
    }

    public function woocommerce_missing_notice() {
        ?>
        <div class="notice notice-error">
            <p><?php esc_html_e( 'Mahn & Rot erfordert WooCommerce.', 'mahn-rot-rechnungsbearbeiter' ); ?></p>
        </div>
        <?php
    }

    /**
     * Instead of add_menu_page(), we use add_submenu_page() with 'woocommerce' as the parent slug.
     */
    public function register_admin_menu() {
        add_submenu_page(
            'woocommerce',                    // Parent slug (the WooCommerce menu)
            'M&R Inkasso Bestellungen',              // Page title
            'M&R Inkasso',                     // Menu title in WooCommerce
            'manage_woocommerce',             // Capability
            'mahn-rot-orders',                // Menu slug
            array( $this, 'render_orders_page' ) // Callback
        );
    }

    /**
     * Render the main page (includes credentials form & order list).
     */
    public function render_orders_page() {

        // (1) Show success message if we have "?mahn_rot_sent="
        if ( isset( $_GET['mahn_rot_sent'] ) ) {
            $sent_count = intval( $_GET['mahn_rot_sent'] );
            ?>
            <div class="notice notice-success is-dismissible">
                <p><?php printf( esc_html__( 'Bestellung(en) erfolgreich an Mahn & Rot versendet: %d', 'mahn-rot-rechnungsbearbeiter' ), $sent_count ); ?></p>
            </div>
            <?php
        }

        // (2) Get stored API credentials
        $api_user = get_option( 'mahn_rot_api_user', '' );
        $api_pass = get_option( 'mahn_rot_api_pass', '' );
        ?>
        <div class="wrap">
            <h1><?php esc_html_e('M&R Inkasso', 'mahn-rot-rechnungsbearbeiter'); ?></h1>

            <!-- Toggle button for API credentials -->
            <button class="button" id="toggle-api-credentials" type="button">
                <?php esc_html_e('Zugangsdaten Mahn & Rot', 'mahn-rot-rechnungsbearbeiter'); ?>
            </button>
                
            <div id="api-credentials-form" style="display:none;">
                 <p><?php esc_html_e('Damit Sie Mahn & Rot Inkasso für Ihren WooCommerce Shop nutzen können registrieren Sie sich bitte auf https://www.mahn-rot.de/mandant und geben anschließend unten Ihre Zugangsdaten ein.', 'mahn-rot-rechnungsbearbeiter'); ?></p>
                <div id= "forborder" style="margin-top:15px;border:1px solid #ccc; padding:10px;">
                <h2><?php esc_html_e('Mahn & Rot Zugangsdaten', 'mahn-rot-rechnungsbearbeiter'); ?></h2>
                <form method="post" action="<?php echo esc_url( admin_url('admin-post.php') ); ?>">
                    <input type="hidden" name="action" value="save_mahn_rot_api_settings" />
                    <?php wp_nonce_field( 'mahn_rot_save_api_settings_nonce' ); ?>

                    <table class="form-table">
                        <tr>
                            <th>
                                <label for="mahn_rot_api_user"><?php esc_html_e('E-Mail', 'mahn-rot-rechnungsbearbeiter'); ?></label>
                            </th>
                            <td>
                                <input
                                    type="text"
                                    name="mahn_rot_api_user"
                                    id="mahn_rot_api_user"
                                    value="<?php echo esc_attr($api_user); ?>"
                                    class="regular-text"
                                />
                            </td>
                        </tr>
                        <tr>
                            <th>
                                <label for="mahn_rot_api_pass"><?php esc_html_e('Passwort', 'mahn-rot-rechnungsbearbeiter'); ?></label>
                            </th>
                            <td>
                                <input
                                    type="password"
                                    name="mahn_rot_api_pass"
                                    id="mahn_rot_api_pass"
                                    value="<?php echo esc_attr($api_pass); ?>"
                                    class="regular-text"
                                />
                                <button type="button" class="button" onclick="mahnRotTogglePass()">
                                    <?php esc_html_e('Ein-/Ausblenden', 'mahn-rot-rechnungsbearbeiter'); ?>
                                </button>
                            </td>
                        </tr>
                    </table>

                    <?php submit_button( __( 'Zugangsdaten speichern', 'mahn-rot-rechnungsbearbeiter' ) ); ?>
                </form>
            </div><!-- #api-credentials-form -->
</div>
            <script>
            document.getElementById('toggle-api-credentials').addEventListener('click', function() {
                var div = document.getElementById('api-credentials-form');
                div.style.display = (div.style.display === 'none') ? 'block' : 'none';
            });

            function mahnRotTogglePass() {
                var passField = document.getElementById('mahn_rot_api_pass');
                passField.type = (passField.type === "password") ? "text" : "password";
            }
            </script>

            <?php
            // (3) Render orders list & filter
            $this->render_orders_list();
            ?>
        </div>
        <?php
    }

    /**
     * List only certain statuses + exclude our new "wc-mr-inkasso" so it won't show up again.
     */
    private function render_orders_list() {
        $current_age = isset($_GET['order_age']) ? sanitize_text_field($_GET['order_age']) : '30';
        $order_age_days = intval($current_age);

        $current_page = isset($_GET['paged']) ? max(1, intval($_GET['paged'])) : 1;
        $date_limit = date('Y-m-d', strtotime("-{$current_age} days"));

        // Only these statuses; exclude "wc-mr-inkasso"
        $query_args = array(
            'status'      => array('pending','on-hold','failed','cancelled'),
            'date_created'=> "<{$date_limit}",
            'orderby'     => 'date',
            'order'       => 'DESC',
            'paginate'    => true,
            'page'        => $current_page,
            'limit'       => $this->orders_per_page,
            'return'      => 'ids',
        );

        $results = wc_get_orders($query_args);
        $orders  = $results->orders; // array of IDs
        $max_pages = $results->max_num_pages;

        ?>
<div class="mahn-rot-notice">
    <p>
        <?php esc_html_e('Wählen Sie unten Bestellungen aus und klicken Sie auf "An Mahn & Rot senden". Beachten Sie bitte dabei, dass Sie nur fällige Forderungen übergeben.', 'mahn-rot-rechnungsbearbeiter'); ?>
    </p>

    <p>
        <?php esc_html_e('Nach der Übermittlung ändert sich der Status in WooCommerce auf "M&R Inkasso". Den genauen Status im Inkassoprozess finden Sie dann im Mandanten-Portal unter:', 'mahn-rot-rechnungsbearbeiter'); ?>
        <a href="https://kundenverwaltung.mahn-rot.de" target="_blank">kundenverwaltung.mahn-rot.de</a>
        <?php esc_html_e('einzeln und transparent für jede Bestellung/Forderung aufgelistet.', 'mahn-rot-rechnungsbearbeiter'); ?>
    </p>

    <p>
        <?php esc_html_e('Bei Rückfragen steht Ihnen unser Kundenservice gerne per E-Mail oder Telefon zur Verfügung:', 'mahn-rot-rechnungsbearbeiter'); ?>
    </p>

    <p>
        <strong><?php esc_html_e('E-Mail:', 'mahn-rot-rechnungsbearbeiter'); ?></strong>
        <a href="mailto:service@mahn-rot.de">service@mahn-rot.de</a>
    </p>

    <p>
        <strong><?php esc_html_e('Telefon:', 'mahn-rot-rechnungsbearbeiter'); ?></strong>
        <?php esc_html_e('0241 412 511 – 70', 'mahn-rot-rechnungsbearbeiter'); ?>
    </p>
</div> 




        <!-- Filter form for order age -->
        <form method="get" style="margin-bottom:1em;">
            <input type="hidden" name="page" value="mahn-rot-orders" />
            <label for="order_age"><?php esc_html_e('Filtern Sie nach Bestellalter:', 'mahn-rot-rechnungsbearbeiter'); ?></label>
            <select id="order_age" name="order_age">
                <option value="30" <?php selected($current_age, '30'); ?>><?php esc_html_e('30 Tage oder älter', 'mahn-rot-rechnungsbearbeiter'); ?></option>
                <option value="60" <?php selected($current_age, '60'); ?>><?php esc_html_e('60 Tage oder älter', 'mahn-rot-rechnungsbearbeiter'); ?></option>
                <option value="90" <?php selected($current_age, '90'); ?>><?php esc_html_e('90 Tage oder älter', 'mahn-rot-rechnungsbearbeiter'); ?></option>
            </select>
            <?php submit_button(__('Filter', 'mahn-rot-rechnungsbearbeiter'), 'secondary', '', false); ?>
        </form>

        <form method="post" action="<?php echo esc_url(admin_url('admin-post.php')); ?>">
            <input type="hidden" name="action" value="mahn_rot_send_orders" />
            <?php wp_nonce_field('mahn_rot_send_orders_nonce', 'mahn_rot_nonce'); ?>

            <table class="wp-list-table widefat fixed striped">
                <thead>
                    <tr>
                        <td style="width: 50px;">
                            <input type="checkbox" id="mahn-rot-select-all" />
                        </td>
                        <th><?php esc_html_e('Bestell-ID', 'mahn-rot-rechnungsbearbeiter'); ?></th>
                        <th><?php esc_html_e('Kundenname', 'mahn-rot-rechnungsbearbeiter'); ?></th>
                        <th><?php esc_html_e('Status', 'mahn-rot-rechnungsbearbeiter'); ?></th>
                        <th><?php esc_html_e('Gesamt', 'mahn-rot-rechnungsbearbeiter'); ?></th>
                        <th><?php esc_html_e('Datum', 'mahn-rot-rechnungsbearbeiter'); ?></th>
                    </tr>
                </thead>
                <tbody>
                    <?php
                    if ( ! empty($orders) ):
                        foreach ( $orders as $order_id ):
                            $order = wc_get_order($order_id);
                            if ( ! $order ) {
                                continue;
                            }
                            $order_edit_url = admin_url('admin.php?page=wc-orders&action=edit&id=' . $order_id);

                            $full_name = trim($order->get_billing_first_name() . ' ' . $order->get_billing_last_name());
                            $status = wc_get_order_status_name($order->get_status());
                            $formatted_total = strip_tags(wc_price($order->get_total()));
                            $date_display = $order->get_date_created()
                                ? $order->get_date_created()->date_i18n(get_option('date_format') . ' ' . get_option('time_format'))
                                : '';
                            ?>
                            <tr>
                                <td>
                                    <input type="checkbox" class="mr-checkbox" name="order_ids[]" value="<?php echo esc_attr($order_id); ?>" />
                                </td>
                                <td>
                                    <a href="<?php echo esc_url($order_edit_url); ?>" target="_blank">
                                        <?php echo '#' . esc_html($order_id); ?>
                                    </a>
                                </td>
                                <td><?php echo esc_html($full_name); ?></td>
                                <td><?php echo esc_html($status); ?></td>
                                <td><?php echo esc_html($formatted_total); ?></td>
                                <td><?php echo esc_html($date_display); ?></td>
                            </tr>
                            <?php
                        endforeach;
                    else:
                        ?>
                        <tr>
                            <td colspan="6">
                                <?php esc_html_e('Keine Bestellungen gefunden.', 'mahn-rot-rechnungsbearbeiter'); ?>
                            </td>
                        </tr>
                    <?php endif; ?>
                </tbody>
            </table>

            <?php submit_button(__('An Mahn & Rot senden', 'mahn-rot-rechnungsbearbeiter'), 'primary', 'submit_mr', true, array( 'id' => 'submit_mr_button' ) ); ?>
        </form>

        <?php
        // If multiple pages, show pagination
        if ( $max_pages > 1 ) {
            $this->render_pagination( $current_page, $max_pages, 'any' );
        }
        ?>

        <!-- Disable the "send" button if no checkbox selected -->
        <script>
            // 1) Toggle checkboxes
            const selectAll = document.getElementById('mahn-rot-select-all');
            selectAll.addEventListener('click', function(e) {
                const checkboxes = document.querySelectorAll('.mr-checkbox');
                checkboxes.forEach(cb => { cb.checked = e.target.checked; });
                updateSendButtonState();
            });

            // 2) When any single box changes
            const boxes = document.querySelectorAll('.mr-checkbox');
            boxes.forEach(box => {
                box.addEventListener('change', updateSendButtonState);
            });

            // 3) The "send" button
            const sendButton = document.getElementById('submit_mr_button');

            function updateSendButtonState() {
                // If no checkbox is selected, disable button
                const anyChecked = Array.from(boxes).some(cb => cb.checked);
                sendButton.disabled = !anyChecked;
            }

            // On page load, disable if none is checked
            updateSendButtonState();
        </script>
        <?php
    }

    public function save_api_credentials() {
        check_admin_referer('mahn_rot_save_api_settings_nonce');

        if ( ! current_user_can('manage_woocommerce') ) {
            wp_die('No permission.');
        }

        $user = isset($_POST['mahn_rot_api_user']) ? sanitize_text_field($_POST['mahn_rot_api_user']) : '';
        $pass = isset($_POST['mahn_rot_api_pass']) ? sanitize_text_field($_POST['mahn_rot_api_pass']) : '';

        update_option('mahn_rot_api_user', $user);
        update_option('mahn_rot_api_pass', $pass);

        // Redirect back
        wp_safe_redirect(admin_url('admin.php?page=mahn-rot-orders'));
        exit;
    }

    public function handle_send_orders() {
        if (!isset($_POST['mahn_rot_nonce']) || !wp_verify_nonce($_POST['mahn_rot_nonce'], 'mahn_rot_send_orders_nonce')) {
            wp_die('Security check failed.');
        }
        if (!current_user_can('manage_woocommerce')) {
            wp_die('No permission.');
        }

        $order_ids = isset($_POST['order_ids']) ? (array)$_POST['order_ids'] : [];
        if (empty($order_ids)) {
            wp_safe_redirect(admin_url('admin.php?page=mahn-rot-orders&mahn_rot_sent=0'));
            exit;
        }

        // Get saved credentials
        $api_user = get_option('mahn_rot_api_user', '');
        $api_pass = get_option('mahn_rot_api_pass', '');

        if ( empty($api_user) || empty($api_pass) ) {
            wp_die('Mahn & Rot API-Anmeldeinformationen nicht festgelegt. Bitte legen Sie diese zuerst fest.');
        }

        try {
            $mr_api = new MahnRot_API($api_user, $api_pass);
        } catch (Exception $e) {
            wp_die('API Initialization failed: ' . esc_html($e->getMessage()));
        }

        $sent_count = 0;
        foreach ($order_ids as $order_id) {
            $order = wc_get_order($order_id);
            if (!$order) {
                continue;
            }

            // Basic Schuldner data
            $schuldner_data = [
                'kundennummer' => (string)$order->get_user_id(),
                'email'        => $order->get_billing_email(),
                'firma'        => $order->get_billing_company(),
                'vorname'      => $order->get_billing_first_name(),
                'nachname'     => $order->get_billing_last_name(),
                'anrede'       => 'Divers',
                'str'          => $order->get_billing_address_1(),
                'hausnr'       => '',
                'ort'          => $order->get_billing_city(),
                'plz'          => $order->get_billing_postcode(),
                'land'         => $order->get_billing_country(),
                'tel'          => $order->get_billing_phone(),
            ];

            $schuldner_type = $order->get_billing_company()
                ? MahnRot_SchuldnerType::COMPANY
                : MahnRot_SchuldnerType::PRIVATE;

            $schuldner = new MahnRot_Schuldner($schuldner_type, $schuldner_data);
            $akte      = new MahnRot_Akte($schuldner);

            $invoice_num  = 'INV-'.$order->get_order_number();
            $created_date = $order->get_date_created()
                ? $order->get_date_created()->date_i18n('d.m.Y')
                : date('d.m.Y');

            $due_date = date('d.m.Y', strtotime('+7 days', $order->get_date_created()->getTimestamp()));

            $forderung = new MahnRot_Forderung(
                $invoice_num,
                $created_date,
                $due_date,
                round($order->get_total() * 100),
                round($order->get_total_tax() * 100)
            );

            // Optionally attach the WC email .eml or other docs
            $eml_content = MahnRot_Email_Helper::generate_order_eml($order_id);
            if ($eml_content) {
                $eml_doc = new MahnRot_Dokument(
                    "order-{$order_id}.eml",
                    MahnRot_DokumentDocType::Rechnungsdokument,
                    MahnRot_DokumentVisibility::VISIBLE,
                    "Original WooCommerce Email"
                );
                $eml_doc->setData($eml_content);
                $forderung->addDokument($eml_doc);
                error_log("[MAHN ROT] Attached .eml to Forderung for order #{$order_id}. Dokument name: " . $eml_doc->getName());
                error_log("[MAHN ROT] Received EML content for order #{$order_id}. Length: " . strlen($eml_content));
            }

            $akte->addForderung($forderung);

            try {
                // Submit to API
                $mr_api->submitAkte($akte);

                // If success => change status to "wc-mr-inkasso" ("M&R Inkasso")
                $order->update_status( 'wc-mr-inkasso', 'Set to M&R Inkasso after sending to API' );

                $sent_count++;
            } catch (Exception $e) {
                error_log("Failed order $order_id: " . $e->getMessage());
                $order->update_status( 'wc-mr-inkasso', 'Set to M&R Inkasso after sending to API' );
            }
        }

        wp_safe_redirect(admin_url('admin.php?page=mahn-rot-orders&mahn_rot_sent=' . $sent_count));
        exit;
    }

    private function render_pagination( $current_page, $max_pages, $current_status ) {
        $base_url = remove_query_arg( array( 'paged' ) );

        echo '<div class="tablenav-pages">';

        if ( $current_page > 1 ) {
            $prev_page = $current_page - 1;
            $prev_url  = add_query_arg(
                array(
                    'paged'        => $prev_page,
                    'order_status' => $current_status,
                ),
                $base_url
            );
            echo '<a class="button" href="' . esc_url( $prev_url ) . '">&laquo; ' . __( 'Previous', 'mahn-rot-rechnungsbearbeiter' ) . '</a> ';
        }

        echo '<span class="displaying-num">' . sprintf(
            __( 'Page %1$d of %2$d', 'mahn-rot-rechnungsbearbeiter' ),
            $current_page,
            $max_pages
        ) . '</span>';

        if ( $current_page < $max_pages ) {
            $next_page = $current_page + 1;
            $next_url  = add_query_arg(
                array(
                    'paged'        => $next_page,
                    'order_status' => $current_status,
                ),
                $base_url
            );
            echo ' <a class="button" href="' . esc_url( $next_url ) . '">' . __( 'Next', 'mahn-rot-rechnungsbearbeiter' ) . ' &raquo;</a>';
        }

        echo '</div>';
    }
}

new MahnRotRechnungsbearbeiterCustomPage();
